@@ -31,6 +31,7 @@
 #include "Error.h"
 #include "Interpreter.h"
 #include "JIT.h"
+#include "JSArrayIterator.h"
 #include "JSStringBuilder.h"
 #include "JSStringJoiner.h"
 #include "Lookup.h"
@@ -64,6 +65,9 @@ static EncodedJSValue JSC_HOST_CALL arrayProtoFuncMap(ExecState*);
 static EncodedJSValue JSC_HOST_CALL arrayProtoFuncReduce(ExecState*);
 static EncodedJSValue JSC_HOST_CALL arrayProtoFuncReduceRight(ExecState*);
 static EncodedJSValue JSC_HOST_CALL arrayProtoFuncLastIndexOf(ExecState*);
+static EncodedJSValue JSC_HOST_CALL arrayProtoFuncValues(ExecState*);
+static EncodedJSValue JSC_HOST_CALL arrayProtoFuncKeys(ExecState*);
+static EncodedJSValue JSC_HOST_CALL arrayProtoFuncEntries(ExecState*);
 
 }
 
@@ -82,7 +86,7 @@ static inline bool isNumericCompareFunction(ExecState* exec, CallType callType,
     if (error)
         return false;
 
-    return executable->generatedBytecodeForCall().isNumericCompareFunction();
+    return executable->codeBlockForCall()->isNumericCompareFunction();
 }
 
 // ------------------------------ ArrayPrototype ----------------------------
@@ -112,33 +116,35 @@ const ClassInfo ArrayPrototype::s_info = {"Array", &JSArray::s_info, 0, ExecStat
   reduce         arrayProtoFuncReduce         DontEnum|Function 1
   reduceRight    arrayProtoFuncReduceRight    DontEnum|Function 1
   map            arrayProtoFuncMap            DontEnum|Function 1
+  entries        arrayProtoFuncEntries        DontEnum|Function 0
+  keys           arrayProtoFuncKeys           DontEnum|Function 0
 @end
 */
 
-ArrayPrototype* ArrayPrototype::create(ExecState* exec, JSGlobalObject* globalObject, Structure* structure)
+ArrayPrototype* ArrayPrototype::create(VM& vm, JSGlobalObject* globalObject, Structure* structure)
 {
-    ArrayPrototype* prototype = new (NotNull, allocateCell<ArrayPrototype>(*exec->heap())) ArrayPrototype(globalObject, structure);
-    prototype->finishCreation(globalObject);
+    ArrayPrototype* prototype = new (NotNull, allocateCell<ArrayPrototype>(vm.heap)) ArrayPrototype(vm, structure);
+    prototype->finishCreation(vm, globalObject);
     return prototype;
 }
 
 // ECMA 15.4.4
-ArrayPrototype::ArrayPrototype(JSGlobalObject* globalObject, Structure* structure)
-    : JSArray(globalObject->vm(), structure, 0)
+ArrayPrototype::ArrayPrototype(VM& vm, Structure* structure)
+    : JSArray(vm, structure, 0)
 {
 }
 
-void ArrayPrototype::finishCreation(JSGlobalObject* globalObject)
+void ArrayPrototype::finishCreation(VM& vm, JSGlobalObject* globalObject)
 {
-    VM& vm = globalObject->vm();
     Base::finishCreation(vm);
     ASSERT(inherits(info()));
     vm.prototypeMap.addPrototype(this);
+    JSC_NATIVE_FUNCTION(vm.propertyNames->iteratorPrivateName, arrayProtoFuncValues, DontEnum, 0);
 }
 
 bool ArrayPrototype::getOwnPropertySlot(JSObject* object, ExecState* exec, PropertyName propertyName, PropertySlot& slot)
 {
-    return getStaticFunctionSlot<JSArray>(exec, ExecState::arrayPrototypeTable(exec), jsCast<ArrayPrototype*>(object), propertyName, slot);
+    return getStaticFunctionSlot<JSArray>(exec, ExecState::arrayPrototypeTable(exec->vm()), jsCast<ArrayPrototype*>(object), propertyName, slot);
 }
 
 // ------------------------------ Array Functions ----------------------------
@@ -154,7 +160,7 @@ static JSValue getProperty(ExecState* exec, JSObject* obj, unsigned index)
 
 static void putProperty(ExecState* exec, JSObject* obj, PropertyName propertyName, JSValue value)
 {
-    PutPropertySlot slot;
+    PutPropertySlot slot(obj);
     obj->methodTable()->put(obj, exec, propertyName, value, slot);
 }
 
@@ -439,7 +445,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncConcat(ExecState* exec)
         }
         if (i == argCount)
             break;
-        curArg = (exec->argument(i));
+        curArg = exec->uncheckedArgument(i);
         ++i;
     }
     arr->setLength(exec, n);
@@ -481,7 +487,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncPush(ExecState* exec)
 
     if (isJSArray(thisValue) && exec->argumentCount() == 1) {
         JSArray* array = asArray(thisValue);
-        array->push(exec, exec->argument(0));
+        array->push(exec, exec->uncheckedArgument(0));
         return JSValue::encode(jsNumber(array->length()));
     }
     
@@ -493,11 +499,11 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncPush(ExecState* exec)
     for (unsigned n = 0; n < exec->argumentCount(); n++) {
         // Check for integer overflow; where safe we can do a fast put by index.
         if (length + n >= length)
-            thisObj->methodTable()->putByIndex(thisObj, exec, length + n, exec->argument(n), true);
+            thisObj->methodTable()->putByIndex(thisObj, exec, length + n, exec->uncheckedArgument(n), true);
         else {
-            PutPropertySlot slot;
+            PutPropertySlot slot(thisObj);
             Identifier propertyName(exec, JSValue(static_cast<int64_t>(length) + static_cast<int64_t>(n)).toWTFString(exec));
-            thisObj->methodTable()->put(thisObj, exec, propertyName, exec->argument(n), slot);
+            thisObj->methodTable()->put(thisObj, exec, propertyName, exec->uncheckedArgument(n), slot);
         }
         if (exec->hadException())
             return JSValue::encode(jsUndefined());
@@ -767,7 +773,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncSplice(ExecState* exec)
 
     unsigned deleteCount = length - begin;
     if (exec->argumentCount() > 1) {
-        double deleteDouble = exec->argument(1).toInteger(exec);
+        double deleteDouble = exec->uncheckedArgument(1).toInteger(exec);
         if (deleteDouble < 0)
             deleteCount = 0;
         else if (deleteDouble > length - begin)
@@ -800,7 +806,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncSplice(ExecState* exec)
             return JSValue::encode(jsUndefined());
     }
     for (unsigned k = 0; k < additionalArgs; ++k) {
-        thisObj->methodTable()->putByIndex(thisObj, exec, k + begin, exec->argument(k + 2), true);
+        thisObj->methodTable()->putByIndex(thisObj, exec, k + begin, exec->uncheckedArgument(k + 2), true);
         if (exec->hadException())
             return JSValue::encode(jsUndefined());
     }
@@ -825,7 +831,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncUnShift(ExecState* exec)
             return JSValue::encode(jsUndefined());
     }
     for (unsigned k = 0; k < nrArgs; ++k) {
-        thisObj->methodTable()->putByIndex(thisObj, exec, k, exec->argument(k), true);
+        thisObj->methodTable()->putByIndex(thisObj, exec, k, exec->uncheckedArgument(k), true);
         if (exec->hadException())
             return JSValue::encode(jsUndefined());
     }
@@ -1143,7 +1149,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncReduce(ExecState* exec)
         array = asArray(thisObj);
 
     if (exec->argumentCount() >= 2)
-        rv = exec->argument(1);
+        rv = exec->uncheckedArgument(1);
     else if (array && array->canGetIndexQuickly(0)) {
         rv = array->getIndexQuickly(0);
         i = 1;
@@ -1220,7 +1226,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncReduceRight(ExecState* exec)
         array = asArray(thisObj);
     
     if (exec->argumentCount() >= 2)
-        rv = exec->argument(1);
+        rv = exec->uncheckedArgument(1);
     else if (array && array->canGetIndexQuickly(length - 1)) {
         rv = array->getIndexQuickly(length - 1);
         i = 1;
@@ -1306,7 +1312,7 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncLastIndexOf(ExecState* exec)
 
     unsigned index = length - 1;
     if (exec->argumentCount() >= 2) {
-        JSValue fromValue = exec->argument(1);
+        JSValue fromValue = exec->uncheckedArgument(1);
         double fromDouble = fromValue.toInteger(exec);
         if (fromDouble < 0) {
             fromDouble += length;
@@ -1332,4 +1338,22 @@ EncodedJSValue JSC_HOST_CALL arrayProtoFuncLastIndexOf(ExecState* exec)
     return JSValue::encode(jsNumber(-1));
 }
 
+EncodedJSValue JSC_HOST_CALL arrayProtoFuncValues(ExecState* exec)
+{
+    JSObject* thisObj = exec->hostThisValue().toThis(exec, StrictMode).toObject(exec);
+    return JSValue::encode(JSArrayIterator::create(exec, exec->callee()->globalObject()->arrayIteratorStructure(), ArrayIterateValue, thisObj));
+}
+
+EncodedJSValue JSC_HOST_CALL arrayProtoFuncEntries(ExecState* exec)
+{
+    JSObject* thisObj = exec->hostThisValue().toThis(exec, StrictMode).toObject(exec);
+    return JSValue::encode(JSArrayIterator::create(exec, exec->callee()->globalObject()->arrayIteratorStructure(), ArrayIterateKeyValue, thisObj));
+}
+    
+EncodedJSValue JSC_HOST_CALL arrayProtoFuncKeys(ExecState* exec)
+{
+    JSObject* thisObj = exec->hostThisValue().toThis(exec, StrictMode).toObject(exec);
+    return JSValue::encode(JSArrayIterator::create(exec, exec->callee()->globalObject()->arrayIteratorStructure(), ArrayIterateKey, thisObj));
+}
+
 } // namespace JSC
